@@ -29,12 +29,11 @@
 
 """Description of all detectors with a factory to instantiate them"""
 
-
 __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "17/04/2024"
+__date__ = "07/02/2025"
 __status__ = "stable"
 
 import logging
@@ -53,6 +52,7 @@ from .. import utils
 from .. import average
 from ..utils import expand2d, crc32, binning as rebin
 from ..utils.decorators import deprecated
+from ..utils.stringutil import to_eng
 
 logger = logging.getLogger(__name__)
 
@@ -121,7 +121,6 @@ class Detector(metaclass=DetectorMeta):
                         "_orientation")
     _MUTABLE_ATTRS = ('_mask', '_flatfield', "_darkcurrent", "_pixel_corners")
 
-
     @classmethod
     def factory(cls, name: str, config: Union[None, str, Dict[str, Any]]=None):
         """
@@ -143,6 +142,8 @@ class Detector(metaclass=DetectorMeta):
         if isinstance(name, Detector):
             # It's already a detector
             return name
+        elif name is None:
+            return None
 
         if os.path.isfile(name):
             # It's a filename
@@ -223,10 +224,10 @@ class Detector(metaclass=DetectorMeta):
             self._pixel1 = float(pixel1)
         if pixel2:
             self._pixel2 = float(pixel2)
-        if (max_shape is None) and ("MAX_SHAPE" in dir(self.__class__)):
-            self.max_shape = tuple(self.MAX_SHAPE)
+        if max_shape is None:
+            self.max_shape = tuple(self.MAX_SHAPE) if "MAX_SHAPE" in dir(self.__class__) else None
         else:
-            self.max_shape = max_shape
+            self.max_shape = tuple(max_shape)
         self.shape = self.max_shape
         self._binning = (1, 1)
         self._mask = False
@@ -249,7 +250,7 @@ class Detector(metaclass=DetectorMeta):
 
         orientation = Orientation(orientation or self.ORIENTATION or 3)
         if (orientation < 0) or (orientation > 4):
-            raise RuntimeError("Unsupported orientation: "+orientation.__doc__)
+            raise RuntimeError("Unsupported orientation: " + orientation.__doc__)
         self._orientation = orientation
 
     def __repr__(self):
@@ -261,7 +262,7 @@ class Detector(metaclass=DetectorMeta):
         if (self._pixel1 is None) or (self._pixel2 is None):
             return "Undefined detector"
         else:
-            txt += f"\t PixelSize= {self._pixel1:.3e}, {self._pixel2:.3e} m"
+            txt += f"\t PixelSize= {to_eng(self._pixel1)}m, {to_eng(self._pixel2)}m"
         if self.orientation:
             txt += f"\t {self.orientation.name} ({self.orientation.value})"
         return txt
@@ -408,7 +409,7 @@ class Detector(metaclass=DetectorMeta):
             if dx.shape == self.max_shape:
                 origin = numpy.atleast_3d(numpy.outer(numpy.ones(self.shape[0]), numpy.arange(self.shape[1])) + dx)
                 corners = numpy.array([0., 0., 1., 1.])  # this is specific to X alias direction2, A and B are on the same X,
-                positions2 = self._pixel2 * (origin + corners[numpy.newaxis, numpy.newaxis, :])
+                positions2 = self._pixel2 * (origin + corners[numpy.newaxis, numpy.newaxis,:])
                 self._pixel_corners[..., 2] = positions2
 
             elif dx.shape == tuple(i + 1 for i in self.max_shape):
@@ -427,7 +428,7 @@ class Detector(metaclass=DetectorMeta):
             # Reset a regular grid, uniform_pixel is not necessary True due to y
             origin = numpy.atleast_3d(numpy.outer(numpy.ones(self.shape[0]), numpy.arange(self.shape[1])))
             corners = numpy.array([0., 0., 1., 1.])  # this is specific to X alias direction2, A and B are on the same X,
-            positions2 = self._pixel2 * (origin + corners[numpy.newaxis, numpy.newaxis, :])
+            positions2 = self._pixel2 * (origin + corners[numpy.newaxis, numpy.newaxis,:])
             self._pixel_corners[..., 2] = positions2
 
     def set_dy(self, dy=None):
@@ -446,15 +447,15 @@ class Detector(metaclass=DetectorMeta):
             if dy.shape == self.max_shape:
                 origin = numpy.atleast_3d(numpy.outer(numpy.arange(self.shape[0]), numpy.ones(self.shape[1])) + dy)
                 corners = numpy.array([0., 1., 1., 0.])  # this is specific to Y alias direction1, A and B are not  the same Y,
-                positions1 = self._pixel1 * (origin + corners[numpy.newaxis, numpy.newaxis, :])
+                positions1 = self._pixel1 * (origin + corners[numpy.newaxis, numpy.newaxis,:])
                 self._pixel_corners[..., 1] = positions1
             elif dy.shape == tuple(i + 1 for i in self.max_shape):
                 d1 = numpy.outer(numpy.arange(self.shape[0] + 1), numpy.ones(self.shape[1] + 1))
                 p1 = (self._pixel1 * (dy + d1))
-                self._pixel_corners[:, :, 0, 1] = p1[:-1, :-1]
-                self._pixel_corners[:, :, 1, 1] = p1[1:, :-1]
-                self._pixel_corners[:, :, 2, 1] = p1[1:, 1:]
-                self._pixel_corners[:, :, 3, 1] = p1[:-1, 1:]
+                self._pixel_corners[:,:, 0, 1] = p1[:-1,:-1]
+                self._pixel_corners[:,:, 1, 1] = p1[1:,:-1]
+                self._pixel_corners[:,:, 2, 1] = p1[1:, 1:]
+                self._pixel_corners[:,:, 3, 1] = p1[:-1, 1:]
             else:
                 raise RuntimeError("detector shape:%s while distortion array: %s" % (self.max_shape, dy.shape))
             self.uniform_pixel = False
@@ -462,7 +463,7 @@ class Detector(metaclass=DetectorMeta):
             # Reset a regular grid, uniform_pixel is not necessary True due to x
             origin = numpy.atleast_3d(numpy.outer(numpy.arange(self.shape[0]), numpy.ones(self.shape[1])))
             corners = numpy.array([0., 1., 1., 0.])  # this is specific to Y alias direction1, A and B are not  the same Y,
-            positions1 = self._pixel1 * (origin + corners[numpy.newaxis, numpy.newaxis, :])
+            positions1 = self._pixel1 * (origin + corners[numpy.newaxis, numpy.newaxis,:])
             self._pixel_corners[..., 1] = positions1
 
     def reset_pixel_corners(self):
@@ -606,7 +607,7 @@ class Detector(metaclass=DetectorMeta):
         if center:
             shape1 = self.shape[0] - 1
             shape2 = self.shape[1] - 1
-        else: #corner
+        else:  # corner
             shape1 = self.shape[0]
             shape2 = self.shape[1]
 
@@ -654,14 +655,14 @@ class Detector(metaclass=DetectorMeta):
             if d1.ndim == 2:
                 if center:
                     self.shape = d1.shape
-                else: #corner
-                    self.shape = tuple(i-1 for i in d1.shape)
+                else:  # corner
+                    self.shape = tuple(i - 1 for i in d1.shape)
         elif "ndim" in dir(d2):
             if d2.ndim == 2:
                 if center:
                     self.shape = d2.shape
-                else: #corner
-                    self.shape = tuple(i-1 for i in d2.shape)
+                else:  # corner
+                    self.shape = tuple(i - 1 for i in d2.shape)
 
         if center:
             # avoid += It modifies in place then segfaults
@@ -763,20 +764,20 @@ class Detector(metaclass=DetectorMeta):
                     # d2 = expand2d(r2, self.shape[0] + 1, True)
                     p1, p2, p3 = self.calc_cartesian_positions(center=False)
                     self._pixel_corners = numpy.zeros((self.shape[0], self.shape[1], 4, 3), dtype=numpy.float32)
-                    self._pixel_corners[:,:, 0, 1] = p1[:-1, :-1]
-                    self._pixel_corners[:,:, 0, 2] = p2[:-1, :-1]
-                    self._pixel_corners[:,:, 1, 1] = p1[1:, :-1]
-                    self._pixel_corners[:,:, 1, 2] = p2[1:, :-1]
+                    self._pixel_corners[:,:, 0, 1] = p1[:-1,:-1]
+                    self._pixel_corners[:,:, 0, 2] = p2[:-1,:-1]
+                    self._pixel_corners[:,:, 1, 1] = p1[1:,:-1]
+                    self._pixel_corners[:,:, 1, 2] = p2[1:,:-1]
                     self._pixel_corners[:,:, 2, 1] = p1[1:, 1:]
                     self._pixel_corners[:,:, 2, 2] = p2[1:, 1:]
                     self._pixel_corners[:,:, 3, 1] = p1[:-1, 1:]
                     self._pixel_corners[:,:, 3, 2] = p2[:-1, 1:]
                     if p3 is not None:
                         # non flat detector
-                        self._pixel_corners[:, :, 0, 0] = p3[:-1, :-1]
-                        self._pixel_corners[:, :, 1, 0] = p3[1:, :-1]
-                        self._pixel_corners[:, :, 2, 0] = p3[1:, 1:]
-                        self._pixel_corners[:, :, 3, 0] = p3[:-1, 1:]
+                        self._pixel_corners[:,:, 0, 0] = p3[:-1,:-1]
+                        self._pixel_corners[:,:, 1, 0] = p3[1:,:-1]
+                        self._pixel_corners[:,:, 2, 0] = p3[1:, 1:]
+                        self._pixel_corners[:,:, 3, 0] = p3[:-1, 1:]
         if correct_binning and self._pixel_corners.shape[:2] != self.shape:
             return self._rebin_pixel_corners()
         else:
@@ -793,10 +794,10 @@ class Detector(metaclass=DetectorMeta):
                 raise RuntimeError("Cannot unbin an image ")
             assert self.CORNERS == 4, "Only valid with quadrilateral pixels"
             pixel_corners = numpy.zeros((self.shape[0], self.shape[1], 4, 3), dtype=numpy.float32)
-            pixel_corners[:, :, 0,:] = self._pixel_corners[::r0, ::r1, 0, :]
-            pixel_corners[:, :, 1,:] = self._pixel_corners[r0 - 1::r0, ::r1, 1, :]
-            pixel_corners[:, :, 2,:] = self._pixel_corners[r0 - 1::r0, r1 - 1::r1, 2, :]
-            pixel_corners[:, :, 3,:] = self._pixel_corners[::r0, r1 - 1::r1, 3, :]
+            pixel_corners[:,:, 0,:] = self._pixel_corners[::r0,::r1, 0,:]
+            pixel_corners[:,:, 1,:] = self._pixel_corners[r0 - 1::r0,::r1, 1,:]
+            pixel_corners[:,:, 2,:] = self._pixel_corners[r0 - 1::r0, r1 - 1::r1, 2,:]
+            pixel_corners[:,:, 3,:] = self._pixel_corners[::r0, r1 - 1::r1, 3,:]
             return pixel_corners
         else:
             return self._pixel_corners
@@ -905,12 +906,13 @@ class Detector(metaclass=DetectorMeta):
             return True
 
         if not self.force_pixel:
-            if shape != self.max_shape:
+            if shape == self.max_shape:
+                self._binning = 1, 1
+                return True
+            else:
                 logger.warning("guess_binning is not implemented for %s detectors!\
                  and image size %s is wrong, expected %s!" % (self.name, shape, self.shape))
                 return False
-            self._binning = 1, 1
-            return True
         elif self.max_shape:
             bin1 = self.max_shape[0] // shape[0]
             bin2 = self.max_shape[1] // shape[1]
@@ -947,6 +949,25 @@ class Detector(metaclass=DetectorMeta):
 #        logger.debug("Detector.calc_mask is not implemented for generic detectors")
         return None
 
+    def get_dummies(self, img):
+        """Calculate the actual dummy value from dtype of the img
+
+        :param img: numpy array (or actually its dtype)
+        :return: actual (dummy, delta_dummy) values as data_d (i.e. float32)
+        """
+        if self.dummy is None:
+            return None, None
+        if isinstance(img, numpy.ndarray):
+            dtype = numpy.dtype(img.dtype)
+        else:
+            dtype = numpy.dtype(img)
+        actual_dummy = numpy.float32(numpy.dtype(img.dtype).type(numpy.int64(self.dummy)))
+        if self.delta_dummy is None:
+            actual_delta_dummy = numpy.finfo("float32").eps
+        else:
+            actual_delta_dummy = numpy.float32(self.delta_dummy)
+        return actual_dummy, actual_delta_dummy
+
     def dynamic_mask(self, img):
         """Calculate the dynamic mask for the given image.
 
@@ -978,7 +999,6 @@ class Detector(metaclass=DetectorMeta):
         dynamic_mask = numpy.logical_or(static_mask, dummy_mask, out=static_mask)
         return dynamic_mask.astype(numpy.int8)
 
-
     ############################################################################
     # Few properties
     ############################################################################
@@ -989,7 +1009,7 @@ class Detector(metaclass=DetectorMeta):
                     self._mask = self.calc_mask()  # gets None in worse cases
                     if self._mask is not None:
                         if self._mask.shape != self.shape:
-                            self._mask = rebin(self._mask, self.binning)!=0
+                            self._mask = rebin(self._mask, self.binning) != 0
                         self._mask = numpy.ascontiguousarray(self._mask, numpy.int8)
                         self._mask_crc = crc32(self._mask)
         return self._mask
@@ -1012,8 +1032,9 @@ class Detector(metaclass=DetectorMeta):
 
     def set_maskfile(self, maskfile):
         if fabio:
-            mask = numpy.ascontiguousarray(fabio.open(maskfile).data,
-                                           dtype=numpy.int8)
+            with fabio.open(maskfile) as fimg:
+                mask = numpy.ascontiguousarray(fimg.data,
+                                               dtype=numpy.int8)
             self.set_mask(mask)
             self._maskfile = maskfile
         else:
@@ -1110,7 +1131,8 @@ class Detector(metaclass=DetectorMeta):
         elif len(files) == 1:
             if fabio is None:
                 raise RuntimeError("FabIO is missing")
-            self.set_flatfield(fabio.open(files[0]).data.astype(numpy.float32))
+            with fabio.open(files[0]) as fimg:
+                self.set_flatfield(fimg.data.astype(numpy.float32))
             self.flatfiles = files[0]
         else:
             self.set_flatfield(average.average_images(files, filter_=method, fformat=None, threshold=0))
@@ -1152,7 +1174,8 @@ class Detector(metaclass=DetectorMeta):
         elif len(files) == 1:
             if fabio is None:
                 raise RuntimeError("FabIO is missing")
-            self.set_darkcurrent(fabio.open(files[0]).data.astype(numpy.float32))
+            with fabio.open(files[0]) as fimg:
+                self.set_darkcurrent(fimg.data.astype(numpy.float32))
             self.darkfiles = files[0]
         else:
             self.set_darkcurrent(average.average_images(files, filter_=method, fformat=None, threshold=0))
@@ -1189,6 +1212,7 @@ class Detector(metaclass=DetectorMeta):
         if self._dummy is not None:
             dummy = self._dummy
         return dummy
+
     @dummy.setter
     def dummy(self, value=None):
         self._dummy = value
@@ -1199,6 +1223,7 @@ class Detector(metaclass=DetectorMeta):
         if self._delta_dummy is not None:
             delta_dummy = self._delta_dummy
         return delta_dummy
+
     @delta_dummy.setter
     def delta_dummy(self, value=None):
         self._delta_dummy = value
@@ -1207,6 +1232,18 @@ class Detector(metaclass=DetectorMeta):
     def orientation(self):
         return self._orientation
 
+    @property
+    def origin(self):
+        "What are the coordinnates of the detector's origin ?"
+        if self.orientation == 1:
+            return self.shape
+        elif self.orientation == 2:
+            return (self.shape[0], 0)
+        elif self.orientation == 4:
+            return (0, self.shape[1])
+        # else self.orientation in (0,3):
+        return (0, 0)
+
 
 class NexusDetector(Detector):
     """
@@ -1222,23 +1259,28 @@ class NexusDetector(Detector):
         "_filename",
         "uniform_pixel") + Detector._UNMUTABLE_ATTRS + Detector._MUTABLE_ATTRS
 
-    def __init__(self, filename=None):
-        Detector.__init__(self)
+    def __init__(self, filename=None, orientation=0):
+        Detector.__init__(self, orientation=orientation)
         self.uniform_pixel = True
         self._filename = None
         if filename is not None:
-            self.load(filename)
+            self.load(filename, orientation=orientation)
 
     def __repr__(self):
-        return "%s detector from NeXus file: %s\t PixelSize= %.3e, %.3e m" % \
-            (self.name, self._filename, self._pixel1, self._pixel2)
+        txt = f"{self.name} detector from NeXus file: {self._filename}\t"
+        txt += f"PixelSize= {to_eng(self._pixel1)}m, {to_eng(self._pixel2)}m"
+        if self.orientation:
+            txt += f"\t {self.orientation.name} ({self.orientation.value})"
+        return txt
 
-    def load(self, filename):
+    def load(self, filename, orientation=0):
         """
         Loads the detector description from a NeXus file, adapted from:
         http://download.nexusformat.org/sphinx/classes/base_classes/NXdetector.html
 
         :param filename: name of the file on the disk
+        :param orientation: overwrite the orientation (if provided in the file)
+        :return: None
         """
         if not io.h5py:
             logger.error("h5py module missing: NeXus detectors not supported")
@@ -1252,7 +1294,7 @@ class NexusDetector(Detector):
             if "API_VERSION" in det_grp:
                 self.API_VERSION = det_grp["API_VERSION"][()].decode()
                 api = [int(i) for i in self.API_VERSION.split(".")]
-                if api>=[1,1] and "CORNERS" in det_grp:
+                if api >= [1, 1] and "CORNERS" in det_grp:
                     self.CORNERS = det_grp["CORNERS"][()]
             if "IS_FLAT" in det_grp:
                 self.IS_FLAT = det_grp["IS_FLAT"][()]
@@ -1274,7 +1316,7 @@ class NexusDetector(Detector):
                 self._delta_dummy = det_grp["delta_dummy"][()]
             for what in ("max_shape", "shape"):
                 if what in det_grp:
-                    self.__setattr__(what, tuple(i for i in det_grp[what][()]))
+                    self.__setattr__(what, tuple(int(i) for i in det_grp[what][()]))
             if "mask" in det_grp:
                 self.mask = det_grp["mask"][()]
             if "pixel_corners" in det_grp:
@@ -1290,10 +1332,12 @@ class NexusDetector(Detector):
                     self.mask = numpy.logical_or(previous_mask, new_mask).astype(numpy.int8)
             else:
                 self.uniform_pixel = True
-            if "orientation" in det_grp:
+            if orientation:  # Highest priority
+                self._orientation = Orientation(orientation)
+            elif "orientation" in det_grp:  # Restore what was stored in the HDF5 file
                 self._orientation = Orientation(det_grp["orientation"][()])
-            else:
-                self._orientation = Orientation(3)
+            else:  # Initialize with default value
+                self._orientation = Orientation(self.ORIENTATION or 3)
         # Populate shape and max_shape if needed
         if self.max_shape is None:
             if self.shape is None:
@@ -1381,7 +1425,8 @@ class NexusDetector(Detector):
 
         :return: dict with param for serialization
         """
-        return {"filename": self._filename}
+        return {"filename": self._filename,
+                "orientation": self.orientation or 3}
 
     def getPyFAI(self):
         """
