@@ -26,7 +26,7 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "26/04/2024"
+__date__ = "06/09/2024"
 __status__ = "development"
 
 import logging
@@ -507,11 +507,15 @@ class CscIntegrator2d(CSCIntegrator):
                 std = numpy.sqrt(variance / sum_nrm2)
                 sem[mask] = self.empty
                 std[mask] = self.empty
+                variance = variance.T
+                sem = sem.T
+                std = std.T
+                sum_nrm2 = sum_nrm2.T
             else:
                 variance = std = sem = sum_nrm2 = None
         return Integrate2dtpl(self.bin_centers0, self.bin_centers1,
-                              intensity, sem,
-                              signal, variance, normalization, count, std, sem, sum_nrm2)
+                              intensity.T, sem,
+                              signal.T, variance, normalization.T, count.T, std, sem, sum_nrm2)
 
     integrate_ng = integrate
 
