@@ -25,7 +25,7 @@
 
 __authors__ = ["V. Valls", "J. Kieffer"]
 __license__ = "MIT"
-__date__ = "19/01/2024"
+__date__ = "03/06/2024"
 
 import logging
 import numpy
@@ -37,7 +37,7 @@ import silx.io
 
 import pyFAI.utils
 from .AbstractCalibrationTask import AbstractCalibrationTask
-from pyFAI.azimuthalIntegrator import AzimuthalIntegrator
+from pyFAI.integrator.azimuthal import AzimuthalIntegrator
 from ..utils import unitutils
 from ..model.DataModel import DataModel
 # from ..model.GeometryModel import GeometryModel
@@ -57,8 +57,10 @@ from ...utils import stringutil
 from ..dialog.IntegrationMethodDialog import IntegrationMethodDialog
 from pyFAI import method_registry
 from ..dialog import MessageBox
-from pyFAI.io import ponifile
+from pyFAI.io import ponifile, integration_config
+from pyFAI.worker import Worker
 import pyFAI.geometry
+import json
 
 _logger = logging.getLogger(__name__)
 
@@ -952,6 +954,7 @@ class IntegrationTask(AbstractCalibrationTask):
         self._customMethodButton.clicked.connect(self.__customIntegrationMethod)
 
         self._savePoniButton.clicked.connect(self.__saveAsPoni)
+        self._saveJsonButton.clicked.connect(self.__saveJsonFile)
 
         super()._initGui()
 
@@ -1113,26 +1116,89 @@ class IntegrationTask(AbstractCalibrationTask):
         geometry = self._geometryTabs.geometryModel()
         experimentSettingsModel = model.experimentSettingsModel()
         detector = experimentSettingsModel.detector()
-        pyfaiGeometry = pyFAI.geometry.Geometry(
-            dist=geometry.distance().value(),
-            poni1=geometry.poni1().value(),
-            poni2=geometry.poni2().value(),
-            rot1=geometry.rotation1().value(),
-            rot2=geometry.rotation2().value(),
-            rot3=geometry.rotation3().value(),
-            wavelength=geometry.wavelength().value(),
-            detector=detector
-            )
+
+        writer = ponifile.PoniFile({
+            "dist": geometry.distance().value(),
+            "poni1": geometry.poni1().value(),
+            "poni2":geometry.poni2().value(),
+            "rot1": geometry.rotation1().value(),
+            "rot2": geometry.rotation2().value(),
+            "rot3": geometry.rotation3().value(),
+            "wavelength": geometry.wavelength().value(),
+            "detector":detector.__class__.__name__,
+            "detector_config":detector.get_config(),
+            })
+        comments = [f"Calibrant: {experimentSettingsModel.calibrantModel().calibrant().name}",
+                    f"Image: {experimentSettingsModel.image().filename()}"]
         try:
-            writer = ponifile.PoniFile(pyfaiGeometry)
             with open(filename, "wt") as fd:
-                writer.write(fd)
+                writer.write(fd, comments=comments)
             with poniFile.lockContext():
                 poniFile.setValue(filename)
                 poniFile.setSynchronized(True)
         except Exception as e:
             MessageBox.exception(self, "Error while saving poni file", e, _logger)
 
+    def __saveJsonFile(self):
+        dialog = createSaveDialog(self, "Save as JSON file", json=True)
+        # Disable the warning as the data is append to the file
+        dialog.setOption(qt.QFileDialog.DontConfirmOverwrite, True)
+        model = self.model()
+        jsonFile = model.experimentSettingsModel().jsonFile()
+        previousJsonFile = jsonFile.value()
+        if previousJsonFile is not None:
+            dialog.selectFile(previousJsonFile)
+
+        result = dialog.exec_()
+        if not result:
+            return
+        filename = dialog.selectedFiles()[0]
+        nameFilter = dialog.selectedNameFilter()
+        isJsonFilter = ".json" in nameFilter
+        if isJsonFilter and not filename.endswith(".json"):
+            filename = filename + ".json"
+        with jsonFile.lockContext():
+            jsonFile.setValue(filename)
+
+        geometry = self._geometryTabs.geometryModel()
+        experimentSettingsModel = model.experimentSettingsModel()
+        detector = experimentSettingsModel.detector()
+
+        poni = ponifile.PoniFile({
+            "dist": geometry.distance().value(),
+            "poni1": geometry.poni1().value(),
+            "poni2":geometry.poni2().value(),
+            "rot1": geometry.rotation1().value(),
+            "rot2": geometry.rotation2().value(),
+            "rot3": geometry.rotation3().value(),
+            "wavelength": geometry.wavelength().value(),
+            "detector":detector.__class__.__name__,
+            "detector_config":detector.get_config(),
+            })
+        workerConfig = integration_config.WorkerConfig(poni=poni)
+        workerConfig.nbpt_rad = self.model().integrationSettingsModel().nPointsRadial().value()
+        workerConfig.nbpt_azim = self.model().integrationSettingsModel().nPointsAzimuthal().value()
+        workerConfig.unit = core_units.to_unit(self.model().integrationSettingsModel().radialUnit().value())
+        workerConfig.method = (self.__method.split, self.__method.algo, self.__method.impl)
+        workerConfig.application = "pyFAI-calib2"
+        if self.__polarizationModel.isEnabled():
+            workerConfig.polarization_factor = self.__polarizationModel.value()
+        else:
+            workerConfig.polarization_description = None
+        workerConfig.mask_file = model.experimentSettingsModel().mask().filename()
+        workerConfig.flat_field = model.experimentSettingsModel().flat().filename()
+        workerConfig.dark_current = model.experimentSettingsModel().dark().filename()
+        image = model.experimentSettingsModel().image().value()
+        if image is not None:
+            workerConfig.shape = numpy.array(image).shape
+        try:
+            workerConfig.save(filename)
+            with jsonFile.lockContext():
+                jsonFile.setValue(filename)
+                jsonFile.setSynchronized(True)
+        except Exception as e:
+            MessageBox.exception(self, "Error while saving json file", e, _logger)
+
     def __updateDisplayedGeometry(self):
         "Called after the fit"
         experimentSettingsModel = self.model().experimentSettingsModel()
