@@ -4,7 +4,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2013-2021 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2013-2025 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -33,18 +33,16 @@ __author__ = "Jérôme Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "05/09/2023"
+__date__ = "18/02/2025"
 __status__ = "development"
 
 import logging
-import json
 import os.path
-import collections
-
+import numpy
 logger = logging.getLogger(__name__)
 
 from silx.gui import qt
-
+from ..dialog.MessageBox import exception
 from ..dialog.DetectorSelectorDialog import DetectorSelectorDialog
 from ..dialog.OpenClDeviceDialog import OpenClDeviceDialog
 from ..dialog.GeometryDialog import GeometryDialog
@@ -61,6 +59,9 @@ from ..utils import validators
 from ...io.ponifile import PoniFile
 from ...io import integration_config
 from ... import method_registry
+from ...containers import PolarizationDescription, ErrorModel
+from ... import detector_factory
+from ...integrator import load_engines
 
 
 class _WorkerModel(object):
@@ -142,14 +143,20 @@ class WorkerConfigurator(qt.QWidget):
         self.normalization_factor.setValidator(doubleOrEmptyValidator)
         self.normalization_factor.setText("1.0")
 
-        for value in [None, "poisson", "azimuthal"]:
-            if value:
-                text = value.capitalize()
-            else:
-                text = ""
+        for value in range(1 + max(ErrorModel)):
+            text = ErrorModel(value).as_str().capitalize()
             self.error_model.addItem(text, value)
         self.error_model.setCurrentIndex(0)
 
+        for value in ["integrate",
+                      "sigma_clip_ng",
+                      ]:
+            text = value
+            self.integrator_name.addItem(text, value)
+        self.integrator_name.setCurrentIndex(0)
+        self.sigmaclip_threshold.setText("5.0")
+        self.sigmaclip_maxiter.setText("5")
+
         self.__configureDisabledStates()
 
         self.setDetector(None)
@@ -164,6 +171,7 @@ class WorkerConfigurator(qt.QWidget):
         self.do_radial_range.clicked.connect(self.__updateDisabledStates)
         self.do_azimuthal_range.clicked.connect(self.__updateDisabledStates)
         self.do_normalization.clicked.connect(self.__updateDisabledStates)
+        self.integrator_name.currentTextChanged.connect(self.__updateDisabledStates)
 
         self.__updateDisabledStates()
 
@@ -182,6 +190,8 @@ class WorkerConfigurator(qt.QWidget):
         self.azimuth_range_max.setEnabled(enabled)
         self.normalization_factor.setEnabled(self.do_normalization.isChecked())
         self.monitor_name.setEnabled(self.do_normalization.isChecked())
+        self.sigmaclip_threshold.setEnabled(self.integrator_name.currentText() == "sigma_clip_ng")
+        self.sigmaclip_maxiter.setEnabled(self.integrator_name.currentText() == "sigma_clip_ng")
 
     def set1dIntegrationOnly(self, only1d):
         """Enable only 1D integration for this widget."""
@@ -209,10 +219,28 @@ class WorkerConfigurator(qt.QWidget):
             return None
         return int(value)
 
-    def getConfig(self):
-        """Read the configuration of the plugin and returns it as a dictionary
+    def getPoni(self):
+        poni = {"wavelength": self.__geometryModel.wavelength().value(),
+                "dist": self.__geometryModel.distance().value(),
+                "poni1": self.__geometryModel.poni1().value(),
+                "poni2": self.__geometryModel.poni2().value(),
+                "rot1": self.__geometryModel.rotation1().value(),
+                "rot2": self.__geometryModel.rotation2().value(),
+                "rot3": self.__geometryModel.rotation3().value()}
+        if self.__detector is not None:
+            poni["detector"] = self.__detector.__class__.__name__
+            poni["detector_config"] = self.__detector.get_config()
 
-        :return: dict with all information.
+        ponifile = PoniFile(poni)
+        return ponifile
+
+    def getPoniDict(self):
+        return self.getPoni().as_dict()
+
+    def getWorkerConfig(self):
+        """Read the configuration of the plugin and returns it as a WorkerConfig instance
+
+        :return: WorkerConfig instance
         """
 
         def splitFiles(filenames):
@@ -226,67 +254,43 @@ class WorkerConfigurator(qt.QWidget):
                 return None
             return [name.strip() for name in filenames.split("|")]
 
-        config = collections.OrderedDict()
-
-        # file-version
-        config["application"] = "pyfai-integrate"
-        config["version"] = 3
-
-        # geometry
-        config["wavelength"] = self.__geometryModel.wavelength().value()
-        config["dist"] = self.__geometryModel.distance().value()
-        config["poni1"] = self.__geometryModel.poni1().value()
-        config["poni2"] = self.__geometryModel.poni2().value()
-        config["rot1"] = self.__geometryModel.rotation1().value()
-        config["rot2"] = self.__geometryModel.rotation2().value()
-        config["rot3"] = self.__geometryModel.rotation3().value()
-
-        # detector
-        if self.__detector is not None:
-            config["detector"] = self.__detector.__class__.__name__
-            config["detector_config"] = self.__detector.get_config()
-
+        wc = integration_config.WorkerConfig(application="pyfai-integrate",
+                                             poni=self.getPoni())
         # pre-processing
-        config["do_mask"] = bool(self.do_mask.isChecked())
-        config["mask_file"] = str_(self.mask_file.text()).strip()
-        config["do_dark"] = bool(self.do_dark.isChecked())
-        config["dark_current"] = splitFiles(self.dark_current.text())
-        config["do_flat"] = bool(self.do_flat.isChecked())
-        config["flat_field"] = splitFiles(self.flat_field.text())
-        config["do_polarization"] = bool(self.do_polarization.isChecked())
-        config["polarization_factor"] = float_(self.polarization_factor.value())
-        config["do_dummy"] = bool(self.do_dummy.isChecked())
-        config["val_dummy"] = self._float("val_dummy", None)
-        config["delta_dummy"] = self._float("delta_dummy", None)
+        if self.do_mask.isChecked():
+            wc.mask_image = str_(self.mask_file.text()).strip()
+        if self.do_dark.isChecked():
+            wc.dark_current_image = splitFiles(self.dark_current.text())
+        if self.do_flat.isChecked():
+            wc.flat_field_image = splitFiles(self.flat_field.text())
+        if self.do_polarization.isChecked():
+            wc.polarization_description = PolarizationDescription(float_(self.polarization_factor.value()), 0.0)
+        if self.do_dummy.isChecked():
+            wc.val_dummy = self._float("val_dummy", None)
+            wc.delta_dummy = self._float("delta_dummy", None)
 
         # integration
-        config["do_2D"] = bool(self.do_2D.isChecked())
-        value = self.__getRadialNbpt()
-        if value is not None:
-            config["nbpt_rad"] = value
-
-        value = self.__getAzimuthalNbpt()
-        if value is not None:
-            config["nbpt_azim"] = value
-
-        config["unit"] = str(self.radial_unit.model().value())
-        config["do_radial_range"] = bool(self.do_radial_range.isChecked())
-        config["radial_range_min"] = self._float("radial_range_min", None)
-        config["radial_range_max"] = self._float("radial_range_max", None)
-        config["do_azimuthal_range"] = bool(self.do_azimuthal_range.isChecked())
-        config["azimuth_range_min"] = self._float("azimuth_range_min", None)
-        config["azimuth_range_max"] = self._float("azimuth_range_max", None)
+        if self.do_2D.isChecked():
+            wc.nbpt_azim = self.__getAzimuthalNbpt()
+        wc.nbpt_rad = self.__getRadialNbpt()
+        wc.unit = to_unit(str(self.radial_unit.model().value()))
+        if self.do_radial_range.isChecked():
+            wc.radial_range = [self._float("radial_range_min", -numpy.inf),
+                               self._float("radial_range_max", numpy.inf)]
+        if self.do_azimuthal_range.isChecked():
+            wc.azimuth_range = [self._float("azimuth_range_min", -numpy.inf),
+                                self._float("azimuth_range_max", numpy.inf)]
 
         # processing-config
-        config["chi_discontinuity_at_0"] = bool(self.chi_discontinuity_at_0.isChecked())
-        config["do_solid_angle"] = bool(self.do_solid_angle.isChecked())
-        config["error_model"] = self.error_model.currentData()
+        wc.chi_discontinuity_at_0 = bool(self.chi_discontinuity_at_0.isChecked())
+        wc.correct_solid_angle = bool(self.do_solid_angle.isChecked())
+        wc.error_model = ErrorModel(self.error_model.currentIndex())
 
         method = self.__method
         if method is not None:
-            config["method"] = method.split, method.algo, method.impl
+            wc.method = (method.split, method.algo, method.impl)
             if method.impl == "opencl":
-                config["opencl_device"] = self.__openclDevice
+                wc.opencl_device = self.__openclDevice
 
         if self.do_normalization.isChecked():
             value = self.normalization_factor.text()
@@ -296,70 +300,42 @@ class WorkerConfigurator(qt.QWidget):
                 except ValueError:
                     value = None
                 if value not in [1.0, None]:
-                    config["normalization_factor"] = value
+                    wc.normalization_factor = value
 
             value = self.monitor_name.text()
             if value != "":
                 value = str(value)
-                config["monitor_name"] = value
+                wc.monitor_name = value
 
-        return config
+        if self.integrator_name.currentText() == "sigma_clip_ng":
+            wc.nbpt_azim = 1
+            wc.integrator_method = "sigma_clip_ng"
+            wc.extra_options = {"thres": float(self.sigmaclip_threshold.text()),
+                                "max_iter": float(self.sigmaclip_maxiter.text()),
+                               }
+        return wc
+
+    def getConfig(self):
+        """Read the configuration of the plugin and returns it as a dictionary
+
+        :return: dict with all information
+        """
+        return self.getWorkerConfig().as_dict()
 
     def setConfig(self, dico):
         """Setup the widget from its description
 
-        :param dico: dictionary with description of the widget
+        :param dico: dictionary|WorkerConfig with description of the widget
         :type dico: dict
         """
-        dico = dico.copy()
-        dico = integration_config.normalize(dico, inplace=True)
+        self.setWorkerConfig(integration_config.WorkerConfig.from_dict(dico, inplace=False))
 
-        version = dico.pop("version")
-        if version >= 2:
-            application = dico.pop("application", None)
-            if application != "pyfai-integrate":
-                logger.error("It is not a configuration file from pyFAI-integrate.")
-        if version > 3:
-            logger.error("Configuration file %d too recent. This version of pyFAI maybe too old to read the configuration", version)
+    def setWorkerConfig(self, wc):
+        """Setup the widget from its description
 
-        # Clean up the GUI
-        self.setDetector(None)
-        self.__geometryModel.wavelength().setValue(None)
-        self.__geometryModel.distance().setValue(None)
-        self.__geometryModel.poni1().setValue(None)
-        self.__geometryModel.poni2().setValue(None)
-        self.__geometryModel.rotation1().setValue(None)
-        self.__geometryModel.rotation2().setValue(None)
-        self.__geometryModel.rotation3().setValue(None)
-
-        # geometry
-        if "wavelength" in dico:
-            value = dico.pop("wavelength")
-            self.__geometryModel.wavelength().setValue(value)
-        if "dist" in dico:
-            value = dico.pop("dist")
-            self.__geometryModel.distance().setValue(value)
-        if "poni1" in dico:
-            value = dico.pop("poni1")
-            self.__geometryModel.poni1().setValue(value)
-        if "poni2" in dico:
-            value = dico.pop("poni2")
-            self.__geometryModel.poni2().setValue(value)
-        if "rot1" in dico:
-            value = dico.pop("rot1")
-            self.__geometryModel.rotation1().setValue(value)
-        if "rot2" in dico:
-            value = dico.pop("rot2")
-            self.__geometryModel.rotation2().setValue(value)
-        if "rot3" in dico:
-            value = dico.pop("rot3")
-            self.__geometryModel.rotation3().setValue(value)
-
-        reader = integration_config.ConfigurationReader(dico)
-
-        # detector
-        detector = reader.pop_detector()
-        self.setDetector(detector)
+        :param wc: WorkerConfig instance with the description of the widget
+        :type dico: WorkerConfig
+        """
 
         def normalizeFiles(filenames):
             """Normalize different versions of the filename list.
@@ -374,58 +350,106 @@ class WorkerConfigurator(qt.QWidget):
             filenames = filenames.strip()
             return filenames
 
-        setup_data = collections.OrderedDict()
-        setup_data["val_dummy"] = lambda a: self.val_dummy.setText(str_(a))
-        setup_data["delta_dummy"] = lambda a: self.delta_dummy.setText(str_(a))
-        setup_data["mask_file"] = lambda a: self.__model.maskFileModel.setFilename(str_(a))
-        setup_data["dark_current"] = lambda a: self.__model.darkFileModel.setFilename(normalizeFiles(a))
-        setup_data["flat_field"] = lambda a: self.__model.flatFileModel.setFilename(normalizeFiles(a))
-        setup_data["polarization_factor"] = self.polarization_factor.setValue
-        setup_data["nbpt_rad"] = lambda a: self.nbpt_rad.setText(str_(a))
-        setup_data["nbpt_azim"] = lambda a: self.nbpt_azim.setText(str_(a))
-        setup_data["chi_discontinuity_at_0"] = self.chi_discontinuity_at_0.setChecked
-        setup_data["radial_range_min"] = lambda a: self.radial_range_min.setText(str_(a))
-        setup_data["radial_range_max"] = lambda a: self.radial_range_max.setText(str_(a))
-        setup_data["azimuth_range_min"] = lambda a: self.azimuth_range_min.setText(str_(a))
-        setup_data["azimuth_range_max"] = lambda a: self.azimuth_range_max.setText(str_(a))
-        setup_data["do_solid_angle"] = self.do_solid_angle.setChecked
-        setup_data["do_dummy"] = self.do_dummy.setChecked
-        setup_data["do_dark"] = self.do_dark.setChecked
-        setup_data["do_flat"] = self.do_flat.setChecked
-        setup_data["do_polarization"] = self.do_polarization.setChecked
-        setup_data["do_mask"] = self.do_mask.setChecked
-        setup_data["do_radial_range"] = self.do_radial_range.setChecked
-        setup_data["do_azimuthal_range"] = self.do_azimuthal_range.setChecked
+        # Clean up the GUI
+        self.setDetector(None)
+        self.__geometryModel.wavelength().setValue(None)
+        self.__geometryModel.distance().setValue(None)
+        self.__geometryModel.poni1().setValue(None)
+        self.__geometryModel.poni2().setValue(None)
+        self.__geometryModel.rotation1().setValue(None)
+        self.__geometryModel.rotation2().setValue(None)
+        self.__geometryModel.rotation3().setValue(None)
 
-        for key, value in setup_data.items():
-            if key in dico and (value is not None):
-                value(dico.pop(key))
+        poni = wc.poni
+        if not isinstance(poni, PoniFile):
+            poni = PoniFile(poni)
 
-        normalizationFactor = dico.pop("normalization_factor", None)
-        monitorName = dico.pop("monitor_name", None)
-        self.__setNormalization(normalizationFactor, monitorName)
+        if poni.wavelength:
+            self.__geometryModel.wavelength().setValue(poni.wavelength)
+        if poni.dist:
+            self.__geometryModel.distance().setValue(poni.dist)
+        if poni.poni1 is not None:
+            self.__geometryModel.poni1().setValue(poni.poni1)
+        if poni.poni2 is not None:
+            self.__geometryModel.poni2().setValue(poni.poni2)
+        if poni.rot1 is not None:
+            self.__geometryModel.rotation1().setValue(poni.rot1)
+        if poni.rot2 is not None:
+            self.__geometryModel.rotation2().setValue(poni.rot2)
+        if poni.rot3 is not None:
+            self.__geometryModel.rotation3().setValue(poni.rot3)
 
-        value = dico.pop("unit", None)
+        # detector
+        if poni.detector is not None:
+            self.setDetector(poni.detector)
+
+        self.val_dummy.setText(str_(wc.val_dummy))
+        self.delta_dummy.setText(str_(wc.delta_dummy))
+        self.__model.maskFileModel.setFilename(str_(wc.mask_image))
+        self.__model.darkFileModel.setFilename(normalizeFiles(wc.dark_current_image))
+        self.__model.flatFileModel.setFilename(normalizeFiles(wc.flat_field_image))
+        if isinstance(wc.polarization_factor, float):
+            self.polarization_factor.setValue(wc.polarization_factor)
+        elif isinstance(wc.polarization_factor, (tuple, list)):
+            self.polarization_factor.setValue(wc.polarization_factor[0])
+        self.nbpt_rad.setText(str_(wc.nbpt_rad))
+        self.nbpt_azim.setText(str_(wc.nbpt_azim))
+        self.chi_discontinuity_at_0.setChecked(wc.chi_discontinuity_at_0)
+        self.radial_range_min.setText(str_(wc.radial_range_min))
+        self.radial_range_max.setText(str_(wc.radial_range_max))
+        self.azimuth_range_min.setText(str_(wc.azimuth_range_min))
+        self.azimuth_range_max.setText(str_(wc.azimuth_range_max))
+        self.do_solid_angle.setChecked(bool(wc.correct_solid_angle))
+        self.do_dummy.setChecked(wc.do_dummy)
+        self.do_dark.setChecked(wc.do_dark)
+        self.do_flat.setChecked(wc.do_flat)
+        self.do_polarization.setChecked(wc.do_polarization)
+        self.do_mask.setChecked(wc.do_mask)
+        self.do_radial_range.setChecked(wc.do_radial_range)
+        self.do_azimuthal_range.setChecked(wc.do_azimuthal_range)
+        self.__setNormalization(wc.normalization_factor, wc.monitor_name)
+
+        value = wc.unit
         if value is not None:
-            unit = to_unit(value)
-            self.radial_unit.model().setValue(unit)
+            self.radial_unit.model().setValue(value)
+        if wc.error_model is not None:
+            self.error_model.setCurrentIndex(int(wc.error_model))
 
-        value = dico.pop("error_model", None)
-        index = self.error_model.findData(value)
-        self.error_model.setCurrentIndex(index)
+        dim = 2 if wc.do_2D else 1
+        method = wc.method
+        target = wc.opencl_device
+        if isinstance(target, list):
+            target = tuple(target)
+
+        if method is None:
+            lngm = load_engines.PREFERED_METHODS_2D[0] if dim == 2 else load_engines.PREFERED_METHODS_1D[0]
+            method = lngm.method
+        elif isinstance(method, (str,)):
+            method = method_registry.Method.parsed(method)
+            method = method.fixed(dim=dim, target=target)
+        elif isinstance(method, (list, tuple)):
+            if len(method) == 3:
+                split, algo, impl = method
+                method = method_registry.Method(dim, split, algo, impl, target)
+            elif 3 < len(method) <= 5:
+                method = method_registry.Method(*method)
+            else:
+                raise TypeError(f"Method size {len(method)} is unsupported, method={method}.")
 
-        method = reader.pop_method()
         self.__setMethod(method)
         self.__setOpenclDevice(method.target)
 
-        self.do_2D.setChecked(method.dim == 2)
+        self.do_2D.setChecked(wc.do_2D)
         if self.__only1dIntegration:
             # Force unchecked
             self.do_2D.setChecked(False)
 
-        if len(dico) != 0:
-            for key, value in dico.items():
-                logger.warning("json key '%s' unused", key)
+        integrator_name = wc.integrator_method or "integrate"
+        self.integrator_name.setCurrentText(integrator_name)
+        if integrator_name.startswith("sigma_clip"):
+            extra_options = wc.extra_options or {}
+            self.sigmaclip_threshold.setText(str(extra_options.get("thres", 5.0)))
+            self.sigmaclip_maxiter.setText(str(extra_options.get("max_iter", 5)))
 
         self.__updateDisabledStates()
 
@@ -550,15 +574,9 @@ class WorkerConfigurator(qt.QWidget):
             return
 
         filename = dialog.selectedFiles()[0]
-        config = self.getConfig()
-        if config.get("wavelength") is None:
-            config.pop("wavelength")
-        from ...detectors import detector_factory
-        detector = detector_factory(config.get("detector"), config.get("detector_config"))
-        from ...geometry import Geometry
-        ai = Geometry(detector=detector)
-        ai.set_config(config)
-        ai.save(filename)
+        poni = PoniFile(data=self.getPoniDict())
+        with open(filename, 'w') as fd:
+            poni.write(fd)
 
     def __maskFileChanged(self):
         model = self.__model.maskFileModel
@@ -583,16 +601,14 @@ class WorkerConfigurator(qt.QWidget):
         if not os.path.isfile(filename):
             logger.error("No such file: %s", filename)
             return
-        with open(filename) as f:
-            config = json.load(f)
-        self.setConfig(config)
+        worker_config = integration_config.WorkerConfig.from_file(filename)
+        self.setWorkerConfig(worker_config)
 
     def loadFromPoniFile(self, ponifile):
         try:
             poni = PoniFile(ponifile)
         except Exception as error:
-            # FIXME: An error has to be displayed in the GUI
-            logger.error("file %s does not look like a poni-file, error %s", ponifile, error)
+            exception(self, f"File {ponifile} does not look like a poni-file", error, logger=logger)
             return
 
         model = self.__geometryModel
