@@ -3,7 +3,7 @@
 #    Project: Azimuthal integration
 #             https://github.com/silx-kit/pyFAI
 #
-#    Copyright (C) 2015-2024 European Synchrotron Radiation Facility, Grenoble, France
+#    Copyright (C) 2015-2025 European Synchrotron Radiation Facility, Grenoble, France
 #
 #    Principal author:       Jérôme Kieffer (Jerome.Kieffer@ESRF.eu)
 #
@@ -31,7 +31,7 @@ __author__ = "Jerome Kieffer"
 __contact__ = "Jerome.Kieffer@ESRF.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "24/04/2024"
+__date__ = "21/02/2025"
 __status__ = "production"
 __docformat__ = 'restructuredtext'
 
@@ -165,18 +165,31 @@ class Nexus(object):
             self.h5.attrs["HDF5_Version"] = h5py.version.hdf5_version
             self.h5.attrs["creator"] = creator or self.__class__.__name__
 
+    def __del__(self):
+        self.close()
+
     def close(self, end_time=None):
         """
-        close the filename and update all entries
+        Close the file and update all entries.
         """
-        if self.mode != "r":
-            end_time = get_isotime(end_time)
-            for entry in self.to_close:
-                entry["end_time"] = end_time
-            self.h5.attrs["file_update_time"] = get_isotime()
-        self.h5.close()
-        if self.file_handle:
-            self.file_handle.close()
+        try:
+            if self.mode != "r":
+                if self.h5:
+                    end_time = get_isotime(end_time)
+                    while self.to_close:
+                        entry = self.to_close.pop()
+                        entry["end_time"] = end_time
+                    self.h5.attrs["file_update_time"] = get_isotime()
+        except Exception as error:
+            sys.stderr.write(f"{type(error)}: {error},\nwhile finalizing Nexus file\n")
+
+        try:
+            if self.h5:
+                self.h5.close()
+            if self.file_handle:
+                self.file_handle.close()
+        except Exception as error:
+            sys.stderr.write(f"Error closing file: {error}\n")
 
     # Context manager for "with" statement compatibility
     def __enter__(self, *arg, **kwarg):
