@@ -32,7 +32,7 @@ __authors__ = ["Jérôme Kieffer"]
 __contact__ = "jerome.kieffer@esrf.eu"
 __license__ = "MIT"
 __copyright__ = "European Synchrotron Radiation Facility, Grenoble, France"
-__date__ = "16/04/2024"
+__date__ = "07/01/2025"
 
 import sys
 import unittest
@@ -95,6 +95,9 @@ from . import test_error_model
 from . import test_units
 from . import test_uncertainties
 from . import test_ring_extraction
+from . import test_fiber_integrator
+from . import test_medfilt_engine
+from . import test_containers
 
 logger = logging.getLogger(__name__)
 
@@ -156,7 +159,9 @@ def suite():
     testsuite.addTest(test_error_model.suite())
     testsuite.addTest(test_uncertainties.suite())
     testsuite.addTest(test_ring_extraction.suite())
-
+    testsuite.addTest(test_fiber_integrator.suite())
+    testsuite.addTest(test_medfilt_engine.suite())
+    testsuite.addTest(test_containers.suite())
     return testsuite
 
 
